@@ -51,6 +51,7 @@ class _Formula:
         kind: Optional[Union[MetricFormulaKind, str]] = None,
         operands: Optional[Union[_SingleSeriesOperand, _MultiSeriesOperand, _DatasetFeatureOperand]] = None,
     ):
+        super().__init__()
         self._name = name
         self._kind = MetricFormulaKind(kind.upper()) if kind else None
         self._operands = operands
@@ -121,13 +122,36 @@ class Trigger:
         threshold_value: Optional[float] = None,
         series_name: Optional[str] = None,
         channel_name: Optional[str] = None,
+        description: Optional[str] = None,
     ):
+        super().__init__()
         self._name = name
         self._severity = severity and AlertSeverityKind(severity.upper())
         self._threshold_position = threshold_position and ThresholdPosition(threshold_position.upper())
         self._threshold_value = threshold_value
         self._series_name = series_name
         self._channel_name = channel_name
+        self._description = description
+
+    def with_description(self, description: str) -> "Trigger":
+        """Add a description to your `Trigger`. Descriptions
+        provided here will be included in the alert message in
+        Slack or PagerDuty. Where possible, we render these
+        descriptions as markdown.
+
+        Parameters
+        ----------
+        description
+            A description of the trigger.
+
+        Returns
+        -------
+        Trigger
+            A copy of your `Trigger` with the new description.
+        """
+        copy = self._copy_with()
+        copy._description = description
+        return copy
 
     def with_name(self, name: str) -> "Trigger":
         copy = self._copy_with()
@@ -196,6 +220,7 @@ class Trigger:
 
     def __hash__(self) -> int:
         name = self._name if self._name else "."
+        desc = self._description if self._description else "."
         severity = str(self._severity) if self._severity else "."
         threshold_position = str(self._threshold_position) if self._threshold_position else "."
         threshold_value = str(self._threshold_value) if self._threshold_value else "."
@@ -203,7 +228,7 @@ class Trigger:
         channel_name = self._channel_name if self._channel_name else "."
 
         trigger_string = (
-            f"trigger.{name}.{severity}.{threshold_position}." f"{threshold_value}.{series_name}.{channel_name}"
+            f"trigger.{name}.{severity}.{threshold_position}.{threshold_value}.{series_name}.{channel_name}.{desc}"
         )
 
         return zlib.crc32(trigger_string.encode())
@@ -213,14 +238,14 @@ P = ParamSpec("P")
 T = TypeVar("T")
 
 
-def _copy_with(function: Callable[P, "Chart"]) -> Callable[P, "Chart"]:
+def _copy_with(function: Callable[Concatenate[Chart, P], "Chart"]) -> Callable[Concatenate[Chart, P], "Chart"]:
     def inner(self: Chart, *args: P.args, **kwargs: P.kwargs) -> Chart:
         copy = deepcopy(self)
         if not self._keep:
-            if self in Chart._registry:
-                Chart._registry.remove(self)
+            if self in Chart.registry:
+                Chart.registry.remove(self)
         return_copy = function(copy, *args, **kwargs)
-        Chart._registry.add(return_copy)
+        Chart.registry.add(return_copy)
         return return_copy
 
     return inner
@@ -232,7 +257,7 @@ class Chart:
     Class describing a single visual metric.
     """
 
-    _registry: ClassVar[Set[Union[str, "Chart"]]] = set()
+    registry: ClassVar[Set["Chart"]] = set()
 
     def __init__(self, name: str, window_period: str = "1h", keep: Optional[bool] = False):
         """Create a chart for monitoring or alerting on the Chalk dashboard.
@@ -267,6 +292,7 @@ class Chart:
         ... )
 
         """
+        super().__init__()
         self._name = name
         self._window_period = window_period
         self._series: List[SeriesBase] = []
@@ -275,7 +301,7 @@ class Chart:
         self._keep = keep
         self._entity_id = None
         self._entity_kind = ChartLinkKind.manual
-        Chart._registry.add(self)
+        Chart.registry.add(self)
 
     @_copy_with
     def with_name(self, name: str) -> "Chart":
@@ -328,12 +354,14 @@ class Chart:
             A copy of your chart with the new name
         """
         if not isinstance(series, SeriesBase):
-            raise ValueError(f"'series' value '{series}' must be a Series object")
+            raise ValueError(f"'series' value '{series}' must be a Series object for Chart '{self._name}'")
         if series._entity_id and self._entity_id and series._entity_id != self._entity_id:
             raise ValueError(
-                f"Chart '{self._name}' is already associated with '{self._entity_id}', but new Series "
-                f"'{series.name}' is associated with '{series._entity_id}'. "
-                f"Series can only be associated with one entity."
+                (
+                    f"Chart '{self._name}' is already associated with '{self._entity_id}', but new Series "
+                    f"'{series.name}' is associated with '{series._entity_id}'. "
+                    f"Series can only be associated with one entity."
+                )
             )
         self._series.append(series)
         if series._entity_id:
@@ -378,7 +406,7 @@ class Chart:
     ) -> "Chart":
         if formula:
             if not isinstance(formula, _Formula):
-                raise ValueError(f"'formula' value '{formula}' must be a Formula object")
+                raise ValueError(f"'formula' value '{formula}' must be a Formula object for Chart '{self._name}'")
             self._formulas.append(formula)
             return self
         if name:
@@ -387,7 +415,10 @@ class Chart:
             new_formula = _Formula(name=name, kind=kind, operands=operands)
             self._formulas.append(new_formula)
             return self
-        raise ValueError("Either a 'name' for a new formula or an existing Formula 'formula' must be supplied")
+        raise ValueError(
+            "Either a 'name' for a new formula or an existing Formula 'formula' "
+            "must be supplied for Chart '{self._name}'"
+        )
 
     @_copy_with
     def with_trigger(
@@ -404,6 +435,7 @@ class Chart:
             ],
         ] = AlertSeverityKind.INFO,
         channel_name: Optional[str] = None,
+        description: Optional[str] = None,
     ) -> "Chart":
         """Attaches a `Trigger` to your `Chart`. Your `Chart` may optionally have one `Trigger`.
 
@@ -427,6 +459,13 @@ class Chart:
                 - `info`
         channel_name
             The owner or email of the trigger.
+        description
+            A description to your `Trigger`. Descriptions
+            provided here will be included in the alert message in
+            Slack or PagerDuty.
+
+            For Slack alerts, you can use the mrkdwn syntax described here:
+            https://api.slack.com/reference/surfaces/formatting#basics
 
         Returns
         -------
@@ -434,7 +473,9 @@ class Chart:
             A copy of your `Chart` with the new trigger.
         """
         if not isinstance(expression.lhs, SeriesBase):
-            raise ValueError(f"Left hand side of expression '{expression.lhs}' must be a Series")
+            raise ValueError(
+                f"Left hand side of expression '{expression.lhs}' must be a Series for Chart '{self._name}'"
+            )
         if self._trigger is not None:
             raise ValueError(
                 f"Trigger is already set for Chart '{self._name}'. Only one alert trigger can be set per chart."
@@ -453,6 +494,7 @@ class Chart:
             threshold_value=expression.rhs,
             series_name=expression.lhs.name,
             channel_name=channel_name,
+            description=description,
         )
         return self
 
@@ -564,3 +606,7 @@ class Chart:
         trigger = str(hash(self._trigger)) if self._trigger else "."
 
         return zlib.crc32(chart_string.encode() + series_hash.encode() + formulas.encode() + trigger.encode())
+
+    @property
+    def name(self):
+        return self._name
