@@ -2144,6 +2144,7 @@ class ChalkClient:
         input_features: Optional[list[str]] = None,
         output_features: Optional[list[str]] = None,
         source_config: Optional[SourceConfig] = None,
+        dependencies: Optional[List[str]] = None,
     ) -> RegisterModelVersionResponse:
         """
         Register a model in the Chalk model registry.
@@ -2181,6 +2182,9 @@ class ChalkClient:
             e.g. `["user.is_spam"]`
         source_config : LocalSourceConfig, S3SourceConfig, HFSourceConfig, optional
             Config to pass credentials to access files from a remote source.
+        dependencies : List[str], optional
+            List of package dependencies needed to run this model.
+            e.g. ["torch==2.7.1", "numpy==1.26.4"]
         Returns
         -------
         ModelVersion
@@ -2190,10 +2194,10 @@ class ChalkClient:
         --------
         Register from Python object:
 
-        >>> client.register_model_version(
+         >>> client.register_model_version(
         ...     name="RiskModel",
-        ...     model=trained_sklearn_model,
-        ...     model_type="pytorch",
+        ...     model=trained_pytorch_model,
+        ...     model_type=ModelType.PYTORCH,
         ... )
 
         Register from local files:
@@ -2204,9 +2208,9 @@ class ChalkClient:
         >>> client.register_model_version(
         ...     name="RiskModel",
         ...     model_paths=["./model.pth"],
-        ...     model_type="pytorch",
-        ...     input_schema=pa.large_string(),
-        ...     output_schema=pa.float32()
+        ...     model_type=ModelType.PYTORCH,
+        ...     input_schema={"content": pa.large_string()},
+        ...     output_schema={"prob": pa.float64()}
         ... )
 
         Register from s3 path:
@@ -2214,7 +2218,7 @@ class ChalkClient:
         >>> client.register_model_version(
         ...     name="RiskModel",
         ...     model_paths=["s3://my-bucket/path/to/model.pth"],
-        ...     model_type="pytorch",
+        ...     model_type=ModelType.PYTORCH,
         ... )
         """
         ...
