@@ -1,16 +1,37 @@
 from __future__ import annotations
 
+import json
+import types
+import typing
 from datetime import date, datetime, time, timedelta
 from decimal import Decimal
 from io import BytesIO
-from typing import Any, Generic, Optional, Protocol, Sequence, Type, TypeVar, Union, cast, overload
+from typing import (
+    Any,
+    Dict,
+    FrozenSet,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Protocol,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    final,
+    overload,
+)
 
 import dateutil.tz
 import numpy as np
 import pyarrow as pa
 import pyarrow.compute as pc
 import pyarrow.feather as pf
-from typing_extensions import Final
+from typing_extensions import Final, get_args, get_origin
 
 from chalk._gen.chalk.arrow.v1 import arrow_pb2 as pb
 from chalk.features._encoding.json import (
@@ -30,7 +51,8 @@ from chalk.features._encoding.pyarrow import (
 )
 from chalk.features._encoding.rich import structure_primitive_to_rich, unstructure_rich_to_primitive
 from chalk.utils.collections import unwrap_annotated_if_needed, unwrap_optional_and_annotated_if_needed
-from chalk.utils.json import TJSON
+from chalk.utils.df_utils import pa_array_to_pl_series
+from chalk.utils.json import JSON, TJSON, is_pyarrow_json_type, pyarrow_json_type
 from chalk.utils.log_with_context import get_logger
 
 _logger = get_logger(__name__)
@@ -129,7 +151,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
             ) from e
         try:
             import polars as pl
-        except ImportError:
+        except ModuleNotFoundError:
             pass
         else:
             del pl
@@ -137,6 +159,39 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
 
     __slots__ = ("_name", "_pyarrow_dtype", "_is_nullable", "_primitive_type", "polars_dtype", "_pyarrow_default")
 
+    def __eq__(self, other: object):
+        if not isinstance(other, PrimitiveFeatureConverter):
+            return NotImplemented
+        if not (
+            self._name == other._name
+            and self._pyarrow_dtype == other._pyarrow_dtype
+            and self._is_nullable == other.is_nullable
+        ):
+            return False
+        if self._pyarrow_default == other._pyarrow_default:
+            # Fast path -- defaults are definitively equal if 1x1 pyarrow arrays of the default value are equal
+            return True
+        else:
+            if self._pyarrow_default is ... or other._pyarrow_default is ...:
+                # Already handled the case where they are both ellipsis above, since ... == ...
+                # If here, then one side has a default, when the other side doesn't
+                return False
+            # Could have false negatives due to nan/null equality
+            # To deal, we'll convert both to polars, and compare. This should (recursively) handle nans/nulls
+            self_pl = pa_array_to_pl_series(self._pyarrow_default)
+            other_pl = pa_array_to_pl_series(other._pyarrow_default)
+            import polars.testing
+
+            try:
+                # Must use the testing API because otherwise nan != nan
+                polars.testing.assert_series_equal(self_pl, other_pl, check_exact=True)
+                return True
+            except AssertionError:
+                return False
+
+    def __hash__(self) -> int:
+        return hash((self._name, self._pyarrow_dtype, self._is_nullable, self._pyarrow_default))
+
     def __getstate__(self):
         # We do NOT include the polars dtype or primitive type on the state; we will reconstruct these from the pyarrow dtype
         # Primitive types might differ if we dynamically construct a class (in the case of structs)
@@ -148,7 +203,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
 
         try:
             import polars as pl
-        except ImportError:
+        except ModuleNotFoundError:
             pass
         else:
             del pl  # unused
@@ -166,22 +221,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
     def from_pyarrow_to_primitive(self, values: Union[pa.Array, pa.ChunkedArray]) -> Sequence[_TPrim]:
         return [_recursively_unwrap(x, self._pyarrow_dtype) for x in values.to_pylist()]
 
-    @classmethod
-    def from_pyarrow_to_protobuf(cls, values: Union[pa.Array, pa.ChunkedArray]) -> list[pb.ScalarValue]:
-        if isinstance(values, pa.ChunkedArray):
-            values = values.combine_chunks()
-        if not isinstance(values, pa.Array):
-            raise TypeError(f"Expected PyArrow `Array`, but got `{type(values).__name__}`")
-        return [FeatureConverter._from_pa_to_pb(x) for x in values]
-
-    def from_primitive_to_protobuf_single(self, value: _TPrim) -> pb.ScalarValue:
-        pa_arr = self.from_primitive_to_pyarrow([value])
-        pb_values = self.from_pyarrow_to_protobuf(pa_arr)
-        if len(pb_values) != 1:
-            raise ValueError(f"Expected exactly one value, but got {len(pb_values)}")
-        return pb_values[0]
-
-    def from_primitive_to_pyarrow(self, value: Sequence[_TPrim]) -> Union[pa.Array, pa.ChunkedArray]:
+    def from_primitive_to_pyarrow(self, value: Iterable[_TPrim]) -> Union[pa.Array, pa.ChunkedArray]:
         # FIXME: Convert recursively
         if pa.types.is_float16(self._pyarrow_dtype):
             return pa.array(np.array([None if x is ... else x for x in value], np.dtype("float16")))
@@ -190,6 +230,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
             list_size: int = self._pyarrow_dtype.list_size
             value_type: pa.DataType = self._pyarrow_dtype.value_type
             if pa.types.is_float16(value_type):
+                value = tuple(value)
                 empty = [None] * list_size
                 # NOTE: Numpy converts both nan and null to nan for individual array elements.
                 # Because pc.if_else doesn't work for float16 arrays, it is difficult to flip the null bits from the python api
@@ -223,25 +264,23 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
 
     def from_json_to_primitive(self, value: Union[TJSON, TPrimitive]) -> _TPrim:
         try:
+            if self._primitive_type == JSON:  # pyright: ignore[reportUnnecessaryComparison]
+                # The JSON is the primitive!
+                return cast(_TPrim, value)
             return cast(_TPrim, structure_json_to_primitive(value, self._primitive_type))
         except (ValueError, TypeError) as e:
-            raise TypeError(f"Could not convert '{value}' to `{self._primitive_type}`") from e
-
-    @classmethod
-    def from_protobuf_to_pyarrow(cls, scalars: list[pb.ScalarValue]) -> pa.Array:
-        pa_scalars = [cls._from_pb_to_pa(x) for x in scalars]
-        py_objects = [x.as_py() for x in pa_scalars]
-        arr_type = pa_scalars[0].type if pa_scalars else None
-        ans = pa.array(py_objects, type=arr_type)
-        assert isinstance(ans, pa.Array)
-        return ans
+            raise TypeError(f"Could not convert '{value}' to `{self._primitive_type}`: {e}") from e
 
     @property
-    def pyarrow_dtype(self):
+    def pyarrow_dtype(self) -> pa.DataType:
         return self._pyarrow_dtype
 
     @property
-    def is_nullable(self):
+    def pyarrow_default(self) -> ellipsis | pa.Array | pa.ChunkedArray:
+        return self._pyarrow_default
+
+    @property
+    def is_nullable(self) -> bool:
         return self._is_nullable
 
     @property
@@ -249,7 +288,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
         return self._primitive_type
 
     @property
-    def has_default(self):
+    def has_default(self) -> bool:
         # primitive default is set <==> rich default is also set or no rich type was provided
         return self._pyarrow_default != ...
 
@@ -273,9 +312,15 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
         return False
 
     @classmethod
-    def _from_pb_to_pa(cls, pb_value: pb.ScalarValue) -> pa.Scalar:
+    def from_protobuf_to_pyarrow(cls, pb_value: pb.ScalarValue) -> pa.Scalar:
+        if pb_value.HasField("extension_value"):
+            storage_val = cls.from_protobuf_to_pyarrow(pb_value.extension_value.storage_value)
+            logical_type = cls.convert_proto_dtype_to_pa_dtype(
+                pb.ArrowType(extension=pb_value.extension_value.extension_type)
+            )
+            return pa.ExtensionScalar.from_storage(logical_type, storage_val)
         if pb_value.HasField("null_value"):
-            return pa.scalar(value=None, type=cls.convert_proto_dtype_to_pa_dtype(pb_value.null_value))
+            return pa.nulls(1, type=cls.convert_proto_dtype_to_pa_dtype(pb_value.null_value))[0]
         if pb_value.HasField("bool_value"):
             return pa.scalar(pb_value.bool_value, pa.bool_())
         if pb_value.HasField("int8_value"):
@@ -339,7 +384,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                     pa.time32("ms"),
                 )
             raise ValueError(
-                f"Unsupported time32 value - missing fields `time32_second_value` and `time32_millisecond_value`"
+                "Unsupported time32 value - missing fields `time32_second_value` and `time32_millisecond_value`"
             )
         if pb_value.HasField("time64_value"):
             if pb_value.time64_value.HasField("time64_microsecond_value"):
@@ -367,7 +412,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                     pa.time64("ns"),
                 )
             raise ValueError(
-                f"Unsupported time64 value - missing fields `time64_microsecond_value` and `time64_nanosecond_value`"
+                "Unsupported time64 value - missing fields `time64_microsecond_value` and `time64_nanosecond_value`"
             )
         if pb_value.HasField("timestamp_value"):
             tz_str = pb_value.timestamp_value.timezone
@@ -387,12 +432,14 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                     datetime.fromtimestamp(nanoseconds / 1_000_000_000, tz=tz), pa.timestamp("ns", tz=tz_str)
                 )
             raise ValueError(
-                f"Unsupported protobuf timestamp value - missing fields `time_second_value`, "
-                + "`time_millisecond_value`, `time_microsecond_value`, and `time_nanosecond_value`"
+                (
+                    "Unsupported protobuf timestamp value - missing fields `time_second_value`, "
+                    "`time_millisecond_value`, `time_microsecond_value`, and `time_nanosecond_value`"
+                )
             )
         if pb_value.HasField("struct_value"):
             name_to_pa_scalar = {
-                field.name: cls._from_pb_to_pa(field_value)
+                field.name: cls.from_protobuf_to_pyarrow(field_value)
                 for field, field_value in zip(pb_value.struct_value.fields, pb_value.struct_value.field_values)
             }
             # TODO: Add test using `v.as_py()` values that evaluate to 0 for testing `if (o := v.as_py()) is not None`
@@ -421,7 +468,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
         raise ValueError(f"Unsupported Protobuf type: {pb_value}")
 
     @classmethod
-    def _from_pa_to_pb(cls, value: pa.Scalar) -> pb.ScalarValue:
+    def from_pyarrow_to_protobuf(cls, value: pa.Scalar) -> pb.ScalarValue:
         if value.as_py() is None:
             return pb.ScalarValue(null_value=cls.convert_pa_dtype_to_proto_dtype(value.type))
         if pa.types.is_null(value.type):
@@ -571,7 +618,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
             fields = []
             field_values = []
             for name, pa_scalar in value.items():
-                pb_scalar = cls._from_pa_to_pb(pa_scalar)
+                pb_scalar = cls.from_pyarrow_to_protobuf(pa_scalar)
                 fields.append(
                     pb.Field(
                         name=name,
@@ -703,6 +750,10 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                 item_type=item_field,
                 keys_sorted=dtype.map.keys_sorted,
             )
+        if dtype.HasField("extension"):
+            if dtype.extension.name == "arrow.json":
+                return pyarrow_json_type()
+            raise ValueError(f"Unsupported extension type: '{dtype.extension.name}'")
 
         raise ValueError(f"Could not convert the protobuf type `{dtype}` to a pyarrow dtype")
 
@@ -747,13 +798,13 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
         elif pa.types.is_date64(dtype):
             return pb.ArrowType(date64=pb.EmptyMessage())
         elif dtype == pa.time32("s"):
-            return pb.ArrowType(time32=pb.TimeUnit.TIME_UNIT_SECOND)
+            return pb.ArrowType(time32=pb.TIME_UNIT_SECOND)
         elif dtype == pa.time32("ms"):
-            return pb.ArrowType(time32=pb.TimeUnit.TIME_UNIT_MILLISECOND)
+            return pb.ArrowType(time32=pb.TIME_UNIT_MILLISECOND)
         elif dtype == pa.time64("us"):
-            return pb.ArrowType(time64=pb.TimeUnit.TIME_UNIT_MICROSECOND)
+            return pb.ArrowType(time64=pb.TIME_UNIT_MICROSECOND)
         elif dtype == pa.time64("ns"):
-            return pb.ArrowType(time64=pb.TimeUnit.TIME_UNIT_NANOSECOND)
+            return pb.ArrowType(time64=pb.TIME_UNIT_NANOSECOND)
         elif isinstance(dtype, pa.TimestampType):
             return pb.ArrowType(timestamp=pb.Timestamp(time_unit=UNIT_TO_PROTOBUF[dtype.unit], timezone=dtype.tz))
         elif isinstance(dtype, pa.DurationType):
@@ -809,7 +860,15 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                     ),
                 ),
             )
-        raise TypeError(f"Could not convert the pyarrow dtype {dtype} to a protobuf message")
+        elif isinstance(dtype, pa.ExtensionType):
+            return pb.ArrowType(
+                extension=pb.Extension(
+                    name=dtype.extension_name,
+                    storage_type=cls.convert_pa_dtype_to_proto_dtype(dtype.storage_type),
+                )
+            )
+        else:
+            raise TypeError(f"Could not convert the pyarrow dtype {dtype} to a protobuf message")
 
     @staticmethod
     def _serialize_pa_decimal_to_pb(value: Union[pa.Decimal128Scalar, pa.Decimal256Scalar]) -> pb.ScalarValue:
@@ -927,7 +986,7 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
                 + "`list_value`, `large_list_value`, `map_value`, or `fixed_size_list_value`"
             )
 
-        if not isinstance(value, pb.ScalarListValue):
+        if not isinstance(value, pb.ScalarListValue):  # pyright: ignore[reportUnnecessaryIsInstance]
             raise TypeError(f"Unsupported list value type: {type(value).__name__}")
 
         if value.arrow_data:
@@ -957,6 +1016,10 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
         if d is None:
             return None
         if isinstance(dtype, pa.MapType):
+            if isinstance(d, pa.MapScalar):
+                # If the value is already wrapped as an Arrow scalar, unwrap it.
+                d = {k: v for k, v in d.as_py()}
+
             if not isinstance(d, dict):
                 raise TypeError(f"Expected a `dict` but got: {type(d).__name__}")
             return [{"key": k, "value": cls._recursive_dict_to_list_of_dicts(v, dtype.item_type)} for k, v in d.items()]
@@ -974,6 +1037,18 @@ class PrimitiveFeatureConverter(Generic[_TPrim]):
             return res
         return d
 
+    def from_primitive_to_protobuf(
+        self,
+        value: _TPrim,
+    ) -> pb.ScalarValue:
+        if isinstance(value, pa.Scalar):
+            # This value is alreay a scalar.
+            as_scalar = value
+        else:
+            as_arr = self.from_primitive_to_pyarrow([value])
+            as_scalar = as_arr[0]
+        return self.from_pyarrow_to_protobuf(as_scalar)
+
 
 class TEncoder(Protocol[_TPrimCo, _TRichCon]):
     def __call__(self, value: _TRichCon, /) -> _TPrimCo:
@@ -985,6 +1060,63 @@ class TDecoder(Protocol[_TPrimCon, _TRichCo]):
         ...
 
 
+def _encode_json(t: JSON) -> str:
+    return json.dumps(t)
+
+
+def _decode_json(s: str) -> JSON:
+    return cast(JSON, json.loads(s))
+
+
+def _to_old_style_type(origin: object):
+    if origin is list:
+        return List
+    elif origin is tuple:
+        return Tuple
+    elif origin is set:
+        return Set
+    elif origin is frozenset:
+        return FrozenSet
+    elif origin is dict:
+        return Dict
+    elif hasattr(types, "UnionType"):
+        # Only available in python >=3.10
+        if origin is getattr(types, "UnionType"):
+            return Union
+    return origin
+
+
+def canonicalize_typ(x: object):
+    """Canonicalize a type annotation for equality checking.
+    New-style types are replaced with old-style types, and
+    annotated markers are ignored. Specifically:
+
+    - typing.Annotated -> unwrapped
+    - list             -> typing.List
+    - tuple            -> typing.Tuple
+    - set              -> typing.Set
+    - frozenset        -> typing.Frozenset
+    - dict             -> typing.Dict
+    - union            -> typing.Union
+    """
+
+    if x is None:
+        x = type(None)
+    x = unwrap_annotated_if_needed(x)
+    origin, args = get_origin(x), get_args(x)
+    if origin is None:
+        return _to_old_style_type(x)
+    origin = _to_old_style_type(origin)
+    args = tuple(canonicalize_typ(x) for x in args)
+    return origin[args]  # type: ignore -- pyright doesn't understand metaprogramming
+
+
+@final
+class JSONCodec:
+    encode: TEncoder[str, JSON] = _encode_json
+    decode: TDecoder[str, JSON] = _decode_json
+
+
 class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich]):
     """Feature converter that deals with rich types. It supports everything that the primitive feature converter supports.
 
@@ -1046,6 +1178,10 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
         self._primitive_type = pyarrow_to_primitive(pyarrow_dtype, name)
         self._pyarrow_dtype = pyarrow_dtype
         self._is_nullable = is_nullable
+
+        # This field is also set in the super() call, but must be initialized here
+        # because it is also used for error handling inside of `from_rich_to_primitive`.
+        self._name = name
         if rich_default != ...:
             # The missing value strategy doesn't really matter because rich_default is not missing
             primitive_default = self.from_rich_to_primitive(rich_default, missing_value_strategy="allow")
@@ -1067,7 +1203,7 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
             raise ValueError(f"Feature '{self._name}' has no default value")
         return self._rich_default
 
-    def is_rich_valid(self, value: _TRich):
+    def is_rich_valid(self, value: _TRich) -> bool:
         """Returns true if value has a valid rich type"""
         try:
             prim = self.from_rich_to_primitive(value, "default_or_error")
@@ -1090,7 +1226,7 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
         value: Union[_TRich, ellipsis, None],
         missing_value_strategy: MissingValueStrategy = "default_or_allow",
     ) -> pb.ScalarValue:
-        return self.from_primitive_to_protobuf_single(self.from_rich_to_primitive(value, missing_value_strategy))
+        return self.from_primitive_to_protobuf(self.from_rich_to_primitive(value, missing_value_strategy))
 
     def from_rich_to_primitive(
         self,
@@ -1138,11 +1274,11 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
         return [self.from_primitive_to_rich(x) for x in self.from_pyarrow_to_primitive(values)]
 
     @property
-    def encoder(self):
+    def encoder(self) -> Optional[TEncoder[_TPrim, _TRich]]:
         return self._encoder
 
     @property
-    def decoder(self):
+    def decoder(self) -> Optional[TDecoder[_TPrim, _TRich]]:
         return self._decoder
 
     def _to_primitive(self, val: _TRich) -> _TPrim:
@@ -1159,7 +1295,7 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
                 x = unstructure_rich_to_primitive(val)
             except (TypeError, ValueError) as e:
                 raise TypeError(
-                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}'"
+                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}': {e}"
                 ) from e
             if x is None and not self._is_nullable:
                 raise ValueError(f"Feature '{self._name}' is null, but it cannot be nullable")
@@ -1167,7 +1303,7 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
                 return cast(_TPrim, structure_primitive_to_rich(x, self.primitive_type))
             except (TypeError, ValueError) as e:
                 raise TypeError(
-                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}"
+                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}': {e}"
                 ) from e
         return self._encoder(val)
 
@@ -1189,14 +1325,19 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
                 val = structure_primitive_to_rich(cast(_TPrim, val), cast(Type[_TRich], self.primitive_type))
             except (TypeError, ValueError) as e:
                 raise TypeError(
-                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}'"
+                    f"Could not convert '{val}' to `{self.primitive_type}` for feature '{self._name}': {e}"
                 ) from e
         if self._decoder is None:
             try:
                 return structure_primitive_to_rich(cast(_TPrim, val), self._rich_type)
             except (TypeError, ValueError) as e:
-                raise TypeError(f"Could not convert '{val}' to `{self._rich_type}` for feature '{self._name}'") from e
-        if isinstance(val, unwrap_optional_and_annotated_if_needed(self._rich_type)):
+                raise TypeError(
+                    f"Could not convert '{val}' to `{self._rich_type}` for feature '{self._name}': {e}"
+                ) from e
+        # is_pyarrow_json_type is only needed to handle python 3.8
+        if is_pyarrow_json_type(self.pyarrow_dtype) or isinstance(
+            val, unwrap_optional_and_annotated_if_needed(self._rich_type)
+        ):
             return cast(_TRich, val)
         if val is None:
             # If the value is None, then we won't call the custom converter, since those likely cannot handle null values
@@ -1210,3 +1351,17 @@ class FeatureConverter(PrimitiveFeatureConverter[_TPrim], Generic[_TPrim, _TRich
     def from_json_to_rich(self, value: TJSON) -> _TRich:
         prim_val = self.from_json_to_primitive(value)
         return self.from_primitive_to_rich(prim_val)
+
+    def has_nontrivial_rich_type(self) -> bool:
+        if self._encoder is not None or self._decoder is not None:
+            return True
+
+        prim_canonical = canonicalize_typ(self.primitive_type)
+        rich_canonical = canonicalize_typ(self.rich_type)
+        if self.is_nullable:
+            # Primitive type is based off of the pyarrow dtype, which doesn't know about nullability
+            # So Optional[str] will become just 'str' and needs to be re-wrapped in an Optional[] to compare w/ the rich type
+            # This is mainly a hack for re-creating python Feature objects from serialize proto features (e.g. for running notebook-defined resolvers)
+            # We can remove this once we support encoding more information about the rich type itself in the feature proto.
+            prim_canonical = typing.Optional[prim_canonical]
+        return prim_canonical != rich_canonical
