@@ -281,18 +281,32 @@ def _parse_agg_function_call(expr: Underscore | None) -> Tuple[str, Underscore,
     if aggregation not in supported_aggs:
         raise ChalkParseError(f"aggregation should be one of {', '.join(supported_aggs)}")
 
-    if aggregation in ("approx_top_k", "min_by_n", "max_by_n"):
+    opts = FrozenOrderedSet()
+    if aggregation == "approx_top_k":
         # Special arg validation for approx_top_k, first_k, and last_k.
         if len(call_expr._chalk__args) > 0:
             raise ChalkParseError("should not have any positional arguments")
         elif {"k"} != call_expr._chalk__kwargs.keys():
             raise ChalkParseError("expecting exactly one required keyword argument 'k'")
-        elif (argtype := type(call_expr._chalk__kwargs.get("k"))) != int:
-            raise ChalkParseError(f"expecting 'int' type argument for 'k', but received arg of type '{argtype}'")
+        elif not isinstance(call_expr._chalk__kwargs.get("k"), int):
+            raise ChalkParseError(
+                f"expecting 'int' type argument for 'k', but received arg of type '{type(call_expr._chalk__kwargs.get('k'))}'"
+            )
+        opts = FrozenOrderedSet(call_expr._chalk__kwargs.items())
+    elif aggregation in ("min_by_n", "max_by_n"):
+        if len(call_expr._chalk__kwargs) > 0:
+            raise ChalkParseError("should not have any keyword arguments")
+        if len(call_expr._chalk__args) != 2:
+            raise ChalkParseError("expecting exactly two positional arguments, 'by' and 'k'")
+        if not isinstance(call_expr._chalk__args[1], int):
+            raise ChalkParseError(
+                f"expecting 'int' type argument for 'k', but received arg of type '{type(call_expr._chalk__args[1])}'"
+            )
+        opts = FrozenOrderedSet([("k", call_expr._chalk__args[1])])
     elif len(call_expr._chalk__args) > 0 or len(call_expr._chalk__kwargs) > 0:
         raise ChalkParseError("should not have any arguments or keyword arguments")
 
-    return aggregation, function_attribute._chalk__parent, FrozenOrderedSet(call_expr._chalk__kwargs.items())
+    return aggregation, function_attribute._chalk__parent, opts
 
 
 def _parse_projection(expr: Underscore | None) -> str:
@@ -1009,7 +1023,7 @@ class ChalkImporter:
                 frame = i
         if error_file in self.errors:
             return
-        error_message = f"""{(ex_type and ex_type.__name__) or 'Exception'} at '{error_file}{':' + str(line_number) if line_number is not None else ''}'"""
+        error_message = f"""{(ex_type and ex_type.__name__) or "Exception"} at '{error_file}{":" + str(line_number) if line_number is not None else ""}'"""
         full_traceback = f"""{error_message}:
 {os.linesep.join(traceback.format_tb(ex_traceback)[frame:])}
 {ex_type and ex_type.__name__}: {str(ex_value)}
@@ -1061,6 +1075,7 @@ class ChalkImporter:
         self,
         failed_imports: List[FailedImport],
         diagnostics: List[PublishDiagnosticsParams],
+        additional_diagnostics: Optional[List[PublishDiagnosticsParams]] = None,
     ) -> List[PublishDiagnosticsParams]:
         """
         :param failed_imports: Errors encountered when importing customer code. This method converts them into LSP errors.
@@ -1132,7 +1147,7 @@ class ChalkImporter:
                             )
                         )
 
-        return diagnostics
+        return diagnostics + (additional_diagnostics or [])
 
 
 CHALK_IMPORTER = ChalkImporter()
@@ -1197,7 +1212,7 @@ def import_all_python_files_from_dir(
                 start = time.perf_counter()
                 importlib.import_module(module_path)
                 end = time.perf_counter()
-                _import_logger.debug(f"Imported '{module_path}' in {end-start} seconds")
+                _import_logger.debug(f"Imported '{module_path}' in {end - start} seconds")
             except Exception as e:
                 if not LSPErrorBuilder.promote_exception(e):
                     ex_type, ex_value, ex_traceback = sys.exc_info()
