@@ -35,6 +35,7 @@ from chalk.queries.named_query import NAMED_QUERY_REGISTRY
 from chalk.queries.scheduled_query import CRON_QUERY_REGISTRY
 from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn
 from chalk.sql._internal.sql_source_group import SQLSourceGroup
+from chalk.stores.online_store_config import ONLINE_STORE_CONFIG_REGISTRY
 from chalk.streams import StreamSource
 from chalk.utils.duration import timedelta_to_duration
 
@@ -152,6 +153,7 @@ def export_from_registry() -> export_pb.Export:
         stream_source_registry=StreamSource.registry,
         named_query_registry=NAMED_QUERY_REGISTRY,
         model_reference_registry=MODEL_REFERENCE_REGISTRY,
+        online_store_config_registry=ONLINE_STORE_CONFIG_REGISTRY,
     )
 
     crons: List[CronQuery] = []
