@@ -62,7 +62,7 @@ class StubGenerator:
             item = unwrap_feature(item).fqn
         if isinstance(item, type):
             item = f"{item.__module__}.{item.__name__}"
-        if not isinstance(item, str):
+        if not isinstance(item, str):  # pyright: ignore[reportUnnecessaryIsInstance]
             raise TypeError(f"Invalid type for {item}: {type(item)}")
         item_split = tuple(item.split("."))
         if item_split[0] == "builtins":
@@ -88,7 +88,7 @@ class StubGenerator:
         item = f"__stubgen_{'_'.join(item_split)}"
         return item
 
-    def _register_imports_for_annotation(self, annotation: type) -> Tuple[str, str]:
+    def _register_imports_for_annotation(self, annotation: Any) -> Tuple[str, str]:
         """Register all necessary imports, and return a tuple of (protocol annotation, init annotation)"""
         if annotation == type(None):
             return "None", "None"
@@ -97,7 +97,9 @@ class StubGenerator:
         origin, args = get_origin(annotation), get_args(annotation)
         if origin is None:
             assert args == ()
-            if isinstance(annotation, type) and issubclass(annotation, Vector):
+            if isinstance(annotation, type) and issubclass(
+                annotation, Vector
+            ):  # pyright: ignore[reportUnnecessaryIsInstance]
                 origin_name = self._add_import(Vector, import_as="Vector")
                 return "Vector", "Vector"
             elif annotation.__module__ == "builtins":
@@ -135,6 +137,8 @@ class StubGenerator:
                     include_in_protocol_cls=True,
                     protocol_annotation="DataFrame",
                 )
+            elif isinstance(f.typ._underlying, Feature):  # pyright: ignore[reportPrivateUsage]
+                continue
             else:
                 # Make a best-effort to figure out imports.
                 proto_annotation, init_annotation = self._register_imports_for_annotation(f.typ.parsed_annotation)
@@ -179,6 +183,7 @@ class StubGenerator:
         self._add_import("typing.Iterator", "Iterator")
         self._add_import("chalk.features.feature_set.FeaturesMeta", "FeaturesMeta")
         self._add_import("chalk.features.Features", "Features")
+        self._add_import("chalk.features.TPrimitive", "TPrimitive")
         self._add_import("chalk.features.Feature", "Feature")
         self._add_import("chalk.features.primary.Primary", "Primary")
         self._add_import("chalk.utils.duration.Duration", "Duration")
@@ -287,7 +292,7 @@ class {feature_cls_name}Meta({metaclasses_bases}):"""
                 )
             lines.append(
                 f"""\
-    def __iter__(self) -> Iterator[Feature]: ...
+    def __iter__(self) -> Iterator[Feature[TPrimitive,Any]]: ...
 """
             )
 
